# nestAbundV2.0.r
######################################################
########## Superpop estimation of NEST NUMBER
#######################################################

nestAbund.default <- function() {
#'  sets up global variables with default values
#'@return nothing
#'@seealso \code{\link{nestAbund.runmod}}, \code{\link{nestAbund.sim}}, \code{\link{nestAbund.gui}}
#'@examples
#' nestAbund.default()

  library(tcltk) ; library(MASS)
  VER <<- 'V2.0'
  FirstFound<<-LastPresent<<-Fate<<-Site<<-AgeFound<<-AgeDay1<<-NULL
  Max.Clutch.Size<<-BinCov<<-ContCov<<-NULL

  ContCovName<<-BinCovName<<-'NA'; sout<<-NT<<-DaySampl<<-xdata<<-NULL;
  OccSampl<<-DaysampL1<<-psik<<-fk<<-bk<<-Hk<<-Species<<-NULL;
  modbtn<<-list(); cbvalue<<-list(); ccvalue<<-list() ; wd<<-'.'
  maxit<<-maxbs<<-UUU<<-12; INCUBLEN<<-34; AvgClutch<<-10; Q<<-15;
  sortcol<<-NS<<-CC<<-Ttot<<-daymax<<-DEBUG<<-0
  C1<<-0; BinCovCol<<-ContCovCol<<-0 ; fName<<-'';
  sbar.bname<<-''; sbar.cname<<-'';

  s=Sys.getenv('HOME'); tmp<<-Sys.getenv('TEMP');
  if (nchar(tmp)>0) ftmp<<-paste(tmp,'nestAbund.ini',sep='/')
  if (file.exists(ftmp)) {
    cat('\nloading',ftmp,'\n'); load(ftmp,envir=.GlobalEnv);
  }

  DD<<- -40       # Maximum age at first occasion
  #######################################################
  ## elements for Gauss Hermite quadrature
  #######################################################
  zGH <<- c(-4.49999070730939,-3.66995037340445,-2.9671669279056,-2.32573248617386,
  -1.71999257518649,-1.13611558521092,-0.565069583255576,7.90310504943755e-17,
  0.565069583255576,1.13611558521092,1.71999257518649,2.32573248617386,
  2.9671669279056,3.66995037340445,4.49999070730939)
  wGH <<- c(1.52247580425352e-09,1.05911554771107e-06,0.000100004441232499,
  0.00277806884291277,0.030780033872546,0.158488915795936,0.412028687498899,
  0.564100308726417,0.412028687498898,0.158488915795936,0.030780033872546,
  0.00277806884291277,0.0001000044412325,1.05911554771107e-06,1.52247580425354e-09)

  gui <<- 0; sz <<- 10
  tblfont <<- tkfont.create(family='Lucida Console',size=sz-1)

  i=require('mvtnorm')
  if (!i) {
    j=tkmessageBox(message="package mvtnorm required, do you want to install now?",
      icon = "question", type = "yesno", default = "yes")
    if (tclvalue(j)=="yes") install.packages('mvtnorm')
  }
  library(mvtnorm)
}

#######################################################
### Utility functions to switch from Day to nearest occasion number
#######################################################
adapt_beta<- function(beta,s) {
   n<-max(OccSampl[s,])
   return(beta[1:n]/sum(beta[1:n]))
}
#1 Function  that gives first sampling occasion after the date given in argument
birthocc <- function(firstday,site) {
  zzz <- DaySampl[site,1:NT[site]]-firstday
  return(sum(zzz<0)+1)
}

#2 Function  that gives last sampling occasion before the date given in argument
lastocc <- function(lastday,site) {
  if (lastday<=0) {return(1)}
  else {
  zzz = DaySampl[site,1:NT[site]]-lastday
  return(length(zzz)-sum(zzz>0))
  }
}
#######################################################
## utilitaires
#######################################################
logitgen<-function(p){ n=length(p); ppp=log(p/(1-sum(p[1:(n-1)]))); return(ppp[1:(n-1)])}
invlogitgen<-function(b){
  if (b[1]>50) {    #  try to aviod trouble when all b's are so big
    p=b-b; p[1]=1;  #  that the exp function returns NA's
  } else {
    b=exp(b); p=b*(1/(1+sum(b)));
  }
  return(c(p,(1-sum(p))))
}
LINinvlogitgen<-function(aa){ b=exp(a*xxxstd); p=b*(1/(1+sum(b))); return(c(p,1-sum(p)))}

CIlow <- function(thet) {quantile(thet, probs = c(0.025, 0.975), na.rm = TRUE)[1]}
CIhigh <- function(thet) {quantile(thet, probs = c(0.025, 0.975), na.rm = TRUE)[2]}
#######################################################
## elements for raised cosinus distrib
#######################################################
drcos <- function(x,mu,s) {
  return( (!(abs(x-mu)>s)) * 1/(2*s) * (1 + cos((x-mu)/s*pi)) )
}
prcos <- function(x,mu,s) {
  return( ((x-mu)>=s) + (!(abs(x-mu)>s))*((1+(x-mu)/s + 1/pi*sin((x-mu)/s*pi))/2) )
}

compute_beta <- function(thet_moy_ageday1,thet_sd_ageday1) {
  ### Compute entry probabilities
  #--------------------------------------------------
  thet_beta <- array(0,c(NS,max(NT)))
  thet_beta0=rep(0,Ttot); tb=0
  for (i in 1:Ttot) {
    thet_beta0[i]=pnorm(daysampl1[i],1-thet_moy_ageday1,thet_sd_ageday1)-tb
    tb=tb+thet_beta0[i]
  }
  for (site in 1:NS) {
    totbeta<-sum(thet_beta0[1:OccSampl[site,NT[site]]])
    thet_beta[site,1] <- thet_beta0[1]/totbeta
    for (t in 2:NT[site]) {
      thet_beta[site,t]=sum(thet_beta0[(OccSampl[site,t-1]+1):OccSampl[site,t]])/
                                totbeta
    }
  }
  return(thet_beta)
}

thta.fn <- function(th) {  x=th[pim]
  if (length(fixed)>0) { i=which(!is.na(fixed)); x[i]=fixed[i] }
  return(x)
}
thta.fn.inv <- function(th) { x=NULL
  for (i in 1:max(pim)) { j=which(i==pim)[1]; x[i]=th[j] }
  return(x)
}

mname2pim <- function(mname,v1) {
  l=length(v1); pim<<-1:l; fixed<<-rep(NA,l); k=3;
  s=gsub('.+[(]','',unlist(strsplit(mname,')')))
  if (length(grep('site',  s[1]))<1)  { pim[3+1:NS]<<-k; k=k+1 } else k=NS+3
  if (length(grep('stage', s[1]))<1) { pim[NS+3]<<-1; fixed[NS+3]<<-0 } else {pim[NS+3]<<-k; k=k+1}
  if (length(grep(BinCovName,s[1]))<1 || nchar(BinCovName)<1) { pim[NS+4]<<-1; fixed[NS+4]<<-0 } else {pim[NS+4]<<-k; k=k+1}
  if (length(grep(ContCovName,s[1]))<1 || nchar(ContCovName)<1) { pim[NS+5]<<-1; fixed[NS+5]<<-0 } else {pim[NS+5]<<-k; k=k+1}

  if (length(grep('site',  s[2]))<1)  { pim[NS+5+1:NS]<<-k; k=k+1} else { pim[NS+5+1:NS]<<-k-1+1:NS; k=k+NS}
  if (length(grep('stage', s[2]))<1) { pim[2*NS+6]<<-1; fixed[2*NS+6]<<-0 } else {pim[2*NS+6]<<-k; k=k+1}
  if (length(grep(BinCovName,s[2]))<1 || nchar(BinCovName)<1) { pim[2*NS+7]<<-1; fixed[2*NS+7]<<-0 } else {pim[2*NS+7]<<-k; k=k+1}
  if (length(grep(ContCovName,s[2]))<1 || nchar(ContCovName)<1) { pim[2*NS+8]<<-1; fixed[2*NS+8]<<-0 } else {pim[2*NS+8]<<-k; k=k+1}
  pim[(2*NS+9):l]<<-k:(k+l-2*NS-9)  #  renumber params ocurring after detection params
  if (BinCovCol<1) {   #  if no cattle in data, fix params prop.cattle.site.n
    pim[(3*NS+9):(4*NS+8)]<<-1; fixed[(3*NS+9):(4*NS+8)]<<-0
  }
  if (ContCovCol<1) {   #  if no robel in data, fix params moy_Robel, sd_Robel
    pim[(l-1):l]<<-1; fixed[(l-1):l]<<-c(0,0) #mean(xdata[,ContCovCol]),sd(xdata[,ContCovCol]))
  }
}

getbname <- function(s) {
  a=readLines(s); i=grep('^Prop.',a); s2=''
  if (length(i)>0) s2=strsplit(a[i],' ')
  return(s2[[1]][2])
}

getcname <- function(s) {
  a=readLines(s); i=grep('^moy',a); s2=''
  if (length(i)>0) s2=strsplit(a[i],' ')
  return(s2[[1]][2])
}

sort_aic <- function() {
  l=length(modbtn); np=aic=dlta=ll=mname=fname=dst=bname=cname=NULL
  w="Click model name to view model output."
  if (l>0) {
    for (i in 1:l) {
      x=tclvalue(.Tcl(paste(modbtn[[i]],"cget -text")));
      x=unlist(strsplit(gsub(' +',' ',x),' '))
      sink();
      np[i]=as.numeric(x[2])
      dlta[i]=as.numeric(x[3])
      ll[i]=as.numeric(x[4]) ;   #  ll is neg. log-likelihood
      aic[i]=2*ll[i]+2*np[i]
      mname[i]=gsub(',','',x[length(x)])
      fname[i]=paste('nestAbund_',gsub('+','P',mname[i],fixed=T),'.out',sep='')
      dst[i]='GAU'
      if (length(grep('RCOS',mname[i]))==1) dst[i]='RCOS'
      if (length(grep('TIME',mname[i]))==1) dst[i]='TIME'
      bname[i]=getbname(fname[i])
      cname[i]=getcname(fname[i])
      x=sprintf('%4d %7.2f %7.2f %4s %12s %12s   %s',
               np[i],aic[i],ll[i],dst[i],bname[i],cname[i],mname[i])
    }
    dlta=aic-min(aic)
    aicwgt=exp(-dlta/2); modwgt=aicwgt/sum(aicwgt);
    modlik=modwgt/modwgt[which.min(dlta)]  ;
    #tbl=cbind(sprintf('%4.4d',np),aic,ll,mname); cname=c('npar','AIC','Neg.LL','Model')
    ordr=1:length(aic)
    #if (sortcol>0) { ordr=order(tbl[,sortcol]); w=paste('sorted by',cname[sortcol])}
    for (i in 1:l) {
      j=ordr[i];
      x=sprintf('%4d %7.2f %7.2f %4s %12s %12s   %s',
               np[j],aic[j],ll[j],dst[j],bname[j],cname[j],mname[j])
       cat(i,'>',j,x,'\n')
      tkconfigure(modbtn[[i]],text=x,
      command=eval(substitute(function()
               shell(paste('notepad',fname[k]),wait=F),list(k=j))))
    }
    tkconfigure(statusbar.label,text=w);
  }
}


# Tk inputbox function
inputBox<-function(title_txt='Enter a value',prompt_txt="enter x",dflt="0.5") {
  tt<-tktoplevel(); z<-tclVar(dflt);
  tkwm.title(tt,title_txt)  # Give the window a title
  tkgrid(tklabel(tt,text=prompt_txt),tkentry(tt,width="20",textvariable=z))

  OnOK <- function() { tkdestroy(tt) }
  OK.but <-tkbutton(tt,text="   OK   ",command=OnOK)
  tkbind(tt, "<Return>",OnOK)
  tkgrid(OK.but,columnspan=3)
  tkfocus(tt); tkwait.window(tt); res<-as.numeric(tclvalue(z))
  return(res)
}

